import { defineNuxtModule, createResolver, addComponent, addImports, addTypeTemplate, extendViteConfig } from '@nuxt/kit';

const module = defineNuxtModule({
  meta: {
    name: "nuxt-codemirror",
    configKey: "nuxtCodemirror"
  },
  defaults: {
    // devtools: false,
  },
  setup(_options, _nuxt) {
    const resolver = createResolver(import.meta.url);
    _nuxt.options.alias["#codemirror"] = resolver.resolve("./runtime");
    addComponent({
      name: "NuxtCodeMirror",
      filePath: resolver.resolve("./runtime/components/NuxtCodeMirror.vue")
    });
    addImports({
      name: "useNuxtCodeMirror",
      as: "useNuxtCodeMirror",
      from: resolver.resolve("./runtime/composables/useNuxtCodeMirror.ts")
    });
    addTypeTemplate({
      filename: "nuxt-codemirror.d.ts",
      src: resolver.resolve("./runtime/types/nuxt-codemirror.d.ts")
    });
    extendViteConfig((config) => {
      config.resolve = config.resolve || {};
      config.resolve.alias = config.resolve.alias || {};
      config.resolve.alias["@codemirror/state"] = resolver.resolve(_nuxt.options.rootDir, "node_modules/@codemirror/state");
      config.resolve.alias["@codemirror/view"] = resolver.resolve(_nuxt.options.rootDir, "node_modules/@codemirror/view");
    });
  }
});

export { module as default };
